@@ -4,10 +4,8 @@ from .n0struct_logging import (
     n0print,
     n0debug,
     n0debug_calc,
-    n0error,
 )
 # ******************************************************************************
-# ******************************************************************************
 def findall(current_node: typing.Union[dict, list], seeked_xpath_str: str, raise_exception = True) -> typing.Union[None, dict]:
     if not isinstance(seeked_xpath_str, str):
         raise TypeError(f"seeked_xpath_str: expected str, got other type: {type(seeked_xpath_str)}{str(seeked_xpath_str)}")
@@ -33,10 +31,10 @@ def findfirst(current_node: typing.Union[dict, list], seeked_xpath_str: str, rai
         if raise_exception:
             raise IndexError(f"Not found item {seeked_xpath_str}")
         else:
-            return None,None
+            return None, None
     elif len(found) > 1 and raise_exception:
         raise IndexError(f"Found more that single item {seeked_xpath_str}")  # In case of more than 1 found
-            
+
     found_key = list(found.keys())[0]  # Return first pair in any cases: key, value
     return found_key, found[found_key]
 # ******************************************************************************
@@ -104,7 +102,6 @@ def _findall(
         else:
             # Conditions
             lower_child_index = child_index.lower().replace(' ','')
-            # # n0debug("lower_child_index")
             if lower_child_index == '*':
                 pass  # will be processed lately
             elif lower_child_index.startswith("last()"):
@@ -129,7 +126,7 @@ def _findall(
                     condition_delimiter = '<>'
                 else:
                     raise TypeError(f"Unknown logic condition [{child_index}] in '{str(seeked_xpath_list[0])}'")
-                before_condition,after_condition = child_index.split(condition_delimiter, 1)
+                _before_condition, after_condition = child_index.split(condition_delimiter, 1)
                 value_for_condition = after_condition.strip()
                 if len(value_for_condition) > 1 and \
                    (
@@ -294,5 +291,12 @@ def _findall(
         n0debug("parent_nodes_stack")
         n0print("-"*80)
         raise KeyError(f"Internal error:: looking for {seeked_xpath_list} already found: {found_xpath_list} in {type(parent_node)}'{str(parent_node)}'")
-# ******************************************************************************
-# ******************************************************************************
+
+
+################################################################################
+__all__ = (
+    'findall',
+    'findfirst',
+    '_findall',
+)
+################################################################################
