@@ -2,7 +2,6 @@ import typing
 from .n0struct_n0dict__ import n0dict__
 from .n0struct_logging import n0pretty
 
-# **********************************************************************************************************************
 # *****************************************************************************
 # https://www.w3.org/TR/html4/sgml/entities.html#h-24.4.1
 # https://www.htmlhelp.com/reference/html40/entities/special.html
@@ -67,7 +66,7 @@ class n0dict_(n0dict__):
             raise TypeError(f"Not expected type ({type(value)}) in {path} == {value}")
         return result
     # **************************************************************************
-    def xpath(self, mode: int = None) -> list:  # list[(xpath, value)]
+    def _xpath(self, mode: int = None) -> list:  # list[(xpath, value)]
         """
         Public function: collect elements xpath starts from root
         """
@@ -78,20 +77,20 @@ class n0dict_(n0dict__):
         Public function: collect elements xpath starts from root and print with indents
         """
         result = ""
-        xpath_list = self.xpath(mode)
+        xpath_list = self._xpath(mode)
         if xpath_list:
             xpath_maxlen = max(len(itm[0]) for itm in xpath_list) + 2  # plus 2 chars '"]'
             for itm in xpath_list:
                 result += ("['%-" + str(xpath_maxlen) + "s = %s\n") % \
                             (
-                                itm[0] + "']",  # Don't move to the main
+                                itm[0] + "']",  # Don't move to the main part
                                 ('"' + str(itm[1]) + '"') if itm[1] else "None"
                             )
         return result
     # **************************************************************************
     # XML
     # **************************************************************************
-    def __xml(self, parent: dict, indent: int, inc_indent: int) -> typing.Union[str, None]:
+    def _xml(self, parent: dict, indent: int = 0, indent_size: int = 4) -> typing.Union[str, None]:
         """
         Private function: recursively export n0dict into xml result string
         """
@@ -101,28 +100,37 @@ class n0dict_(n0dict__):
                 if not len(parent.items()):
                     return ""
                 for key, value in parent.items():
-                    if result:
-                        result += "\n"
+                    # if result and (len(parent) > 2 or key not in ("Parm",)):
+                    if key not in ("Parm","ParmCode","Value"):
+                        if result:
+                            result += "\n"
+                        result += f"{' '*indent}"
+
                     if isinstance(value, (list, tuple)):
-                        for i, subitm in enumerate(value):
-                            if i:
-                                result += "\n"
-                            sub_result = self.__xml(subitm, indent+inc_indent, inc_indent)
-                            if not sub_result:
-                                if isinstance(sub_result, str):
-                                    result += " "*indent + f"<{key}></{key}>"
-                                else:
-                                    result += " "*indent + f"<{key}/>"
-                            else:
-                                if '>' in sub_result:
-                                    result += " "*indent + f"<{key}>\n{sub_result}\n" + " "*indent + f"</{key}>"
-                                else:
-                                    result += " "*indent + f"<{key}>{sub_result}</{key}>"
-                    elif isinstance(value, (str, int, float)):
+                        if not value:
+                            result += f"<{key}/>"
+                        else:
+                            result += f"<{key}>"
+                            for i, subitm in enumerate(value):
+                                result += "\n" + self._xml(subitm, indent+indent_size, indent_size)
+                            result += f"\n{' '*indent}</{key}>"
+                    elif isinstance(value, (int, float)):
                         if not key.startswith("@"):
-                            result += " "*indent + f"<{key}>{str(value).translate(html_entities)}</{key}>"
+                            result += f"<{key}>{value}</{key}>"
+                        else:
+                            raise NotImplementedError(f"Export of attibtures ({key}) is not supported yet")
+                    elif isinstance(value, str):
+                        if not key.startswith("@"):
+                            result += f"<{key}>"
+                            if value.lstrip().upper().startswith("<![CDATA[") and value.rstrip().endswith("]]>"):
+                                result += f"\n{' '*(indent+indent_size)}{value}\n{' '*indent}"
+                            else:
+                                result += value.translate(html_entities)
+                            result += f"</{key}>"
+                        else:
+                            raise NotImplementedError(f"Export of attibtures ({key}) is not supported yet")
                     elif isinstance(value, dict):
-                        sub_result = self.__xml(value, indent+inc_indent, inc_indent)
+                        sub_result = self._xml(value, indent+indent_size, indent_size)
 
                         attribs = ""
                         attribs_of_current_key = [(__key[1:], __value) for __key,__value in value.items() if __key.startswith("@")]
@@ -130,79 +138,46 @@ class n0dict_(n0dict__):
                             for __key, __value in attribs_of_current_key:
                                 attribs += f' {__key}="{__value}"'
                         if sub_result:
-                            result += " "*indent + f"<{key}{attribs}>\n{sub_result}\n" + " "*indent + f"</{key}>"
+                            if '\n' in sub_result:
+                                result += f"<{key}{attribs}>\n{sub_result}\n{' '*indent}</{key}>"
+                            else:
+                                if key in ("Parm",):
+                                    result += f"{' '*indent}"
+                                result += f"<{key}{attribs}>{sub_result.lstrip()}</{key}>"
                         else:
-                            result += " "*indent + f"<{key}{attribs}/>"
+                            result += f"<{key}{attribs}/>"
                     elif value is None:
-                        result += " "*indent + f"<{key}/>"
+                        result += f"<{key}/>"
                     else:
-                        raise TypeError(f"__xml(..): Unknown type ({type(value)}) {key} == {value}")
+                        raise TypeError(f"_xml(..): Unknown type ({type(value)}) {key} == {value}")
             elif isinstance(parent, (list, tuple)):
                 if not len(parent):
                     return None
                 for i, itm in enumerate(parent):
                     if i:
                         result += "\n"
-                    result += self.__xml(itm, indent+inc_indent, inc_indent)
+                    result += self._xml(itm, indent+indent_size, indent_size)
             elif isinstance(parent, (str, int, float)):
                 result += str(parent)
             else:
-                raise TypeError(f"__xml(..): Unknown type ({type(parent)}) == {parent}")
+                raise TypeError(f"_xml(..): Unknown type ({type(parent)}) == {parent}")
 
             return result
         else:
             return ""
     # **************************************************************************
-    def to_xml(self, indent: int = 4, encoding: str = "utf-8") -> str:
+    def to_xml(self, indent: int = 0, indent_size: int = 4, encoding: str = "utf-8", quote: str = '"') -> str:
         """
         Public function: export self into xml result string
         """
-        return (f'<?xml version="1.0" encoding="{encoding}"?>\n' if encoding else '') + \
-            self.__xml(self, 0, indent)
+        return (f'<?xml version={quote}1.0{quote} encoding={quote}{encoding}{quote}?>\n' if encoding else '') + \
+            self._xml(self, indent, indent_size)
     # **************************************************************************
     # JSON
     # **************************************************************************
-    def __json(self, parent: dict, indent: int, inc_indent: int) -> str:
-        """
-        Private function: recursively export n0dict into json result string
-        """
-        result = ""
-        for key, value in parent.items():
-            if result:
-                result += ",\n"
-            if isinstance(value, list):
-                sub_result = ""
-                for i, subitm in enumerate(value):
-                    if sub_result:
-                        sub_result += ",\n"
-                    sub_sub_result = self.__json(subitm, (indent+inc_indent)*2, inc_indent)
-                    if sub_sub_result:
-                        if isinstance(subitm, dict):
-                            sub_result += " "*(indent+inc_indent) + "{" + f"\n{sub_sub_result}\n" + " "*(indent+inc_indent) + "}"
-                        elif isinstance(subitm, (list, tuple)):
-                            sub_result += " "*(indent+inc_indent) + f"[\n{sub_sub_result}\n" + " "*(indent+inc_indent) + "]"
-                if sub_result:
-                    result += " "*indent + f'"{key}": [\n{sub_result}\n' + " "*indent + "]"
-                else:
-                    result += " "*indent + f'"{key}": null'
-            elif isinstance(value, str):
-                result += " "*indent + f'"{key}": "{value}"'
-            elif isinstance(value, dict):
-                sub_result = self.__json(value, indent+inc_indent, inc_indent)
-                if sub_result:
-                    result += " "*indent + f'"{key}": ' + "{" + f"\n{sub_result}\n" + " "*indent + "}"
-                else:
-                    result += " "*indent + f'"{key}": null'
-            elif value is None:
-                result += " "*indent + f'"{key}": null'
-            else:
-                raise TypeError(f"Unknown type ({type(value)}) {key} == {value}")
-        return result
-    # **************************************************************************
     def to_json(self,
-                indent: int = 4,
+                indent_size: int = 4,
                 pairs_in_one_line = True,
-                json_convention: bool = True,
                 skip_empty_arrays: bool = False,
                 compress: bool = False,
     ) -> str:
@@ -210,17 +185,24 @@ class n0dict_(n0dict__):
         Public function: export self into json result string
         """
         if compress:
-            indent = 0
+            indent_size = 0
 
-        return n0pretty(self,
-                        show_type=False,
-                        auto_quotes=False,
-                        __quotes='"',
-                        __indent_size=indent,
-                        pairs_in_one_line=pairs_in_one_line,
-                        json_convention=json_convention,
-                        skip_empty_arrays=skip_empty_arrays,
-                        show_item_count=False,
+        return n0pretty(
+            self,
+            show_object_type  = False,
+            indent_size       = indent_size,
+            quote             = '"',
+            pairs_in_one_line = pairs_in_one_line,
+            json_convention   = True,
+            skip_empty_arrays = skip_empty_arrays,
+            skip_simple_types = True,
+            show_item_count   = False,
         )
-# ******************************************************************************
-# ******************************************************************************
+
+
+################################################################################
+__all__ = (
+    'n0dict_',
+    'html_entities',
+)
+################################################################################
