@@ -4,7 +4,7 @@ from .n0struct_utils_find import split_name_index
 from .n0struct_findall import findall as n0struct_findall__findall
 from .n0struct_findall import findfirst as n0struct_findall__findfirst
 from .n0struct_utils import n0eval
-# ******************************************************************************
+
 # ******************************************************************************
 class n0dict__(dict):
     def findall(self, xpath: str, raise_exception: bool = True):
@@ -13,7 +13,7 @@ class n0dict__(dict):
         return n0struct_findall__findfirst(self, xpath, raise_exception)
     # **************************************************************************
     # **************************************************************************
-    def _get(self, xpath: str, raise_exception = True, if_not_found = None, return_lists = True):
+    def _get(self, xpath: str, if_not_found = None, raise_exception = True, return_lists = True):
         """
         Private function:
         return self[where1/where2/.../whereN]
@@ -22,21 +22,26 @@ class n0dict__(dict):
 
         If any of [where1][where2]...[whereN] are not found, exception IndexError will be raised
         """
-        if not xpath:
-            raise IndexError(f"xpath '{xpath}' is not valid")
-        if xpath.startswith('?'):
+        ### if xpath is None or xpath == '':
+        ###    raise IndexError(f"xpath '{xpath}' is not valid")
+
+        if isinstance(xpath, str) and xpath.startswith('?'):
             xpath = xpath[1:]
             raise_exception = False
             if_not_found = ''
-        if any(char in xpath for char in "/["):
-            parent_node, node_name_index, cur_value, xpath_found_str, not_found_xpath_list = self._find(xpath, self, return_lists)
-            if not not_found_xpath_list:
-                return cur_value
-            else:
+
+        if isinstance(xpath, str) and any(char in xpath for char in "/["):
+            try:
+                _parent_node, _node_name_index, cur_value, xpath_found_str, not_found_xpath_list = self._find(xpath, self, return_lists)
+            except (ValueError, IndexError, TypeError, SyntaxError) as caught_ex:
                 if raise_exception:
+                    raise caught_ex
+            else:
+                if not not_found_xpath_list:
+                    return cur_value
+                elif raise_exception:
                     raise IndexError(f"not found '{'/'.join(not_found_xpath_list)}' in '{xpath_found_str}'")
-                else:
-                    return if_not_found
+            return if_not_found
         else:
             try:
                 return super(n0dict__, self).__getitem__(xpath)
@@ -56,7 +61,7 @@ class n0dict__(dict):
 
         If any of [where1][where2]...[whereN] are not found, if_not_found will be returned
         """
-        return self._get(xpath, raise_exception = False, if_not_found = if_not_found)
+        return self._get(xpath, if_not_found = if_not_found, raise_exception = False)
     # **************************************************************************
     # **************************************************************************
     def first(self, xpath: str, if_not_found = None):
@@ -87,7 +92,7 @@ class n0dict__(dict):
         return self._get(xpath, raise_exception = True)
     # **************************************************************************
     # **************************************************************************
-    def __setitem__(self, xpath: str, new_value):
+    def __setitem__(self, xpath: typing.Union[str, int], new_value):
         """
         Public function:
         self[where1/where2/.../whereN] = value
@@ -115,13 +120,13 @@ class n0dict__(dict):
         or commands for creating (convertion into list) new node
             [new()]
         """
-        if xpath.startswith('?'):
+        if isinstance(xpath, str) and xpath.startswith('?'):
             if new_value is None or (isinstance(new_value, str) and new_value == ""):
                 return None
             xpath = xpath[1:]
 
-        if any(char in xpath for char in "/["):
-            parent_node, node_name_index, cur_value, xpath_found_str, not_found_xpath_list = self._find(xpath, self, return_lists = True)
+        if isinstance(xpath, str) and any(char in xpath for char in "/["):
+            parent_node, node_name_index, _cur_value, _xpath_found_str, not_found_xpath_list = self._find(xpath, self, return_lists = True)
             if not_found_xpath_list:
                 parent_node, node_name_index = self._add(parent_node, node_name_index, not_found_xpath_list)
 
@@ -133,7 +138,6 @@ class n0dict__(dict):
             elif isinstance(parent_node, (list, tuple)):
                 if node_name:
                     raise IndexError(f"How is it possible: key '{node_name}' for list ({type(parent_node)})'{parent_node}'?")
-                eval_node_index = n0eval(node_index)
                 parent_node[n0eval(node_index)] = new_value
             else:
                 raise TypeError(f"How is it possible: unknown type of parent node ({type(parent_node)}) of '{parent_node}'")
@@ -144,8 +148,8 @@ class n0dict__(dict):
     # **************************************************************************
     def delete(self, xpath: str, recursively: bool = False) -> n0dict__:
         xpath_list = xpath.split('/')
-        for i,last_xpath_index in enumerate(range(len(xpath_list), 0, -1)):
-            parent_node, node_name_index, cur_value, xpath_found_str, not_found_xpath_list = \
+        for i, last_xpath_index in enumerate(range(len(xpath_list), 0, -1)):
+            parent_node, node_name_index, cur_value, _xpath_found_str, _not_found_xpath_list = \
                 self._find(xpath_list[0:last_xpath_index], self, return_lists=True)
             if i == 0 or (
                 recursively and
@@ -158,9 +162,15 @@ class n0dict__(dict):
                 del parent_node[node_name_index]
         return self
     # **************************************************************************
-    def pop(self, xpath: str, recursively: bool = False) -> typing.Any:
-        result = self[xpath]
-        self.delete(xpath, recursively)
+    def pop(self, xpath: str, if_not_found = None, recursively: bool = False) -> typing.Any:
+        result = if_not_found
+        try:
+            result = self[xpath]
+            self.delete(xpath, recursively)
+        # except KeyError:
+        # except IndexError:
+        except:
+            pass
         return result
     # **************************************************************************
     # **************************************************************************
@@ -182,7 +192,7 @@ class n0dict__(dict):
     def all_not_valid(self, validate):
         return not self._consists_of(validate, True)
     # **************************************************************************
-    def valid(self, node_xpath:str, validate, expected_result_for_error: bool = False, msg:str = None):
+    def valid(self, node_xpath: str, validate, expected_result_for_error: bool = False, msg: str = None):
         """
         :param node_xpath:
             xpath to the node inside self
@@ -195,17 +205,17 @@ class n0dict__(dict):
         :return:
 
         Examples:
-            xml.valid('node/subnode', ["",None], True, "ERROR")
+            xml.valid('node/subnode', ["", None], True, "ERROR")
                 If xml['node/subnode'] is equal "" or None (result of comparising is True), then return ERROR, else ""
-            xml.valid('node/subnode', ["",None], True)
+            xml.valid('node/subnode', ["", None], True)
                 If xml['node/subnode'] is equal "" or None (result of comparising is True), then return False (not valid), else True
             xml.valid('node/subnode', "", True)
                 If xml['node/subnode'] is equal "" (result of comparising is True), then return False (not valid), else True
-            xml.valid('node/subnode', [1,2], False, "ERROR")
+            xml.valid('node/subnode', [1, 2], False, "ERROR")
                 If xml['node/subnode'] is not equal 1 or 2 (result of comparising is False), then return ERROR, else ""
-            xml.valid('node/subnode', [1,2], False)
+            xml.valid('node/subnode', [1, 2], False)
                 If xml['node/subnode'] is not equal 1 or 2 (result of comparising is False), then return False (not valid), else True
-            xml.valid('node/subnode', [1,2])
+            xml.valid('node/subnode', [1, 2])
                 If xml['node/subnode'] is not equal 1 or 2 (result of comparising is False), then return False (not valid), else True
         """
         try:
@@ -216,7 +226,7 @@ class n0dict__(dict):
                 validate_result = node_value in validate
             else:
                 validate_result = node_value == validate
-        except:
+        except Exception:
             validate_result = False
 
         if validate_result == expected_result_for_error:
@@ -230,7 +240,6 @@ class n0dict__(dict):
             else:
                 return ""
     # **************************************************************************
-    # **************************************************************************
     def update_extend(self, other):
         if other is None:
             return self
@@ -259,7 +268,6 @@ class n0dict__(dict):
             raise TypeError("Unexpected type of other: " + str(type(other)))
         return self
     # **************************************************************************
-    # **************************************************************************
     def isExist(self, xpath) -> dict:
         """
         Public function: return empty lists in dict, if self[xpath] exists
@@ -274,49 +282,36 @@ class n0dict__(dict):
             validation_results.update({"difftypes": []})
 
         # TO DO: redo with 'in'
-        try:
+        with contextlib.suppress(Exception):
             if self[xpath]:
                 return validation_results
-        except:
-            pass
+
         validation_results["differences"].append(f"[{xpath}] doesn't exist")
         validation_results["other_unique"].append((xpath, None))
         return validation_results
     # **************************************************************************
-    # **************************************************************************
     def is_exist(self, xpath: str) -> bool:
         """
         Public function: return True, if self[xpath] exists
         """
         # TO DO: redo with 'in'
-        '''
-        try:
-            if self[xpath]:
-                return True
-        except:
-            pass
-        return False
-        '''
         with contextlib.suppress(Exception):
             if self[xpath]:
                 return True
         return False
     # **************************************************************************
-    # **************************************************************************
     def has_all(self, tupple_of_keys: typing.Union[tuple, list]) -> bool:
         for key in tupple_of_keys:
             if key not in self or not self[key]:
                 return False
         return True
     # **************************************************************************
-    # **************************************************************************
     def has_any_of(self, tupple_of_keys: typing.Union[tuple, list]) -> bool:
         for key in tupple_of_keys:
             if key in self:
                 return True
         return False
     # **************************************************************************
-    # **************************************************************************
     def isEqual(self, xpath, value):
         """
         Public function: return empty lists in dict, if self[xpath] == value
@@ -324,16 +319,15 @@ class n0dict__(dict):
         validation_results = self.isExist(xpath)
         if notemptyitems(validation_results):
             return validation_results
-        try:
+
+        with contextlib.suppress(Exception):
             if self[xpath] == value:
                 return []
-        except:
-            pass
+
         validation_results["differences"].append(f"[{xpath}]=='{self[xpath]}' != '{value}'")
         validation_results["not_equal"].append((xpath, (self[xpath], value)))
         return validation_results
     # **************************************************************************
-    # **************************************************************************
     def isTheSame(self, xpath, other_n0dict, other_xpath=None, transformation=lambda x: x):
         """
         Public function: return empty lists in dict, if transformation(self[xpath]) == transformation(other_n0dict[other_xpath])
@@ -343,16 +337,21 @@ class n0dict__(dict):
         validation_results = self.isExist(xpath).update_extend(other_n0dict.isExist(other_xpath))
         if notemptyitems(validation_results):
             return validation_results
-        try:
+
+        with contextlib.suppress(Exception):
             if transformation(self[xpath]) == transformation(other_n0dict[other_xpath]):
                 return validation_results
-        except:
-            # n0print("EXCEPTION in 'if transformation(self[xpath]) == transformation(other_n0dict[other_xpath]):'")
-            pass
+        ## n0print("EXCEPTION in 'if transformation(self[xpath]) == (other_n0dict[other_xpath]):'")
+
         validation_results["differences"].append(
             f"[{xpath}]=='{transformation(self[xpath])}' != [{other_xpath}]=='{transformation(other_n0dict[other_xpath])}'"
         )
         validation_results["not_equal"].append((xpath, (self[xpath], other_n0dict[other_xpath])))
         return validation_results
-# ******************************************************************************
-# ******************************************************************************
+
+
+################################################################################
+__all__ = (
+    'n0dict__',
+)
+################################################################################
