@@ -8,7 +8,6 @@ try:
 except NameError:
     __system_random = random.SystemRandom()
 
-# ******************************************************************************
 # ******************************************************************************
 def rnd(till_not_included: int) -> int:
     """
@@ -23,5 +22,11 @@ def random_from(from_list: list) -> typing.Any:
     :return: from_list[rnd]
     """
     return from_list[rnd(len(from_list))]
-# ******************************************************************************
-# ******************************************************************************
+
+
+################################################################################
+__all__ = (
+    'rnd',
+    'random_from',
+)
+################################################################################
